package appBase;

import java.awt.Font;
import java.util.Enumeration;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.plaf.FontUIResource;

/**
 * The FontManager class provides tha capability of setting
 * the font size for all of the components in a swing
 * application or applet.
 *
 * @author gshute
 */
public class FontManager {
  
  // FIXME - rather than setting all font sizes to be the
  // same, they should be set to scale according to their
  // initial sizes.
  // For example, most fonts in the Metal look and feel are
  // set to size 12 but a few are set to size 10.
  // It might look nicer if those whose initial sizes are 10
  // were set by setFontSize(sz) to size sz*10/12.

  /**
   * FontManager.setFontSize(sz) sets the font size for all
   * swing display components to sz.
   * This method does not by itself change the display.
   * To do that, SwingUtilities.updateComponentTreeUI()
   * should be invoked on the contentPane and, if it exists,
   * the menu bar.
   * <p>
   * BUG:
   * If there are multiple applets in a single web page that
   * use this library then there is no way to control their
   * font sizes individually.
   * They will all share this class so setting the font size
   * for one sets the font size for all.
   * However, SwingUtilities.updateComponentTreeUI() can be
   * invoked on on individual applet content panes and menu
   * bars.
   * The effects of doing this have not been explored.
   * </p>
   * 
   * @param sz
   */
  public static void setFontSize(final float sz) {
    UIDefaults defaults = UIManager.getDefaults();
    Enumeration keys = defaults.keys();
    while (keys.hasMoreElements()) {
      Object key = keys.nextElement();
      Font font = defaults.getFont(key);
      if (font != null) {
        Font scaledFont = font.deriveFont(sz);
        UIManager.put(key, new FontUIResource(scaledFont));
      }
    }
  }

}
