package file;

import java.awt.Dimension;
import java.io.File;
import java.io.FileFilter;
import javax.swing.JLabel;

/**
 *  @author AUTHOR
 */
public class FileGroup {

  private static JLabel label;
  private File directory;
  private FileFilter filter;
  private File[] files;
  private int maxWidth;
  private int maxHeight;

  private void setMaxDimensions() {
    maxWidth = 0;
    maxHeight = 0;
    for (int i = 0; i < files.length; i++) {
      File file = files[i];
      label.setText(file.getName());
      label.setIcon(FileType.getType(file).getIcon());
      Dimension dim = label.getPreferredSize();
      if (dim.width > maxWidth) {
        maxWidth = dim.width;
      }
      if (dim.height > maxHeight) {
        maxHeight = dim.height;
      }
    }
  }

  /**
   *  new FileGroup(ftyp) returns a new FileGroup containing the
   *  files in the current working directory of type ftyp.
   */
  public FileGroup(FileType ftyp) {
    this(new File(System.getProperty("user.dir")), ftyp.getFilter());
  }

  /**
   *  new FileGroup(dir, ftyp) returns a new FileGroup containing the
   *  files in dir of type ftyp.
   */
  public FileGroup(File dir, FileType ftyp) {
    this(dir, ftyp.getFilter());
  }

  /**
   *  new FileGroup(dir, fltr) returns a new FileGroup containing the
   *  files in dir that are accepted by fltr.
   */
  public FileGroup(File dir, FileFilter fltr) {
    if (label == null) {
      label = new JLabel();
      label.setHorizontalAlignment(JLabel.CENTER);
      label.setHorizontalTextPosition(JLabel.CENTER);
      label.setVerticalTextPosition(JLabel.BOTTOM);
    }
    filter = fltr;
    setDirectory(dir);
  }

  public void refresh() {
    System.out.println(directory.toString());
    files = directory.listFiles(filter);
    setMaxDimensions();
  }

  public void setDirectory(File dir) {
    directory = dir;
    refresh();
  }

  public int getFileCount() {
    return files.length;
  }

  public File getFile(int i) {
    return files[i];
  }

  public int getMaxWidth() {
    return maxWidth;
  }

  public int getMaxHeight() {
    return maxHeight;
  }

}  // public class FileGroup

