package file;

// FIXME - we need to add methods for registering actions.
import java.io.File;
import java.io.FileFilter;
import java.util.HashMap;
import java.util.Map;
import javax.swing.Icon;

// Listeners installed in the FileGroup or FilePanel classes
// can invoke these actions.
// The actions should have File or File[] parameters.
/**
 *  @author AUTHOR
 */
public class FileType {

  private static Map<String, FileType> fileTypes;
  
  public static final FileType FOLDER_TYPE =
          new FileType("Folder", "",
          "folder_yellow.png");
  public static final FileType UNKNOWN_TYPE =
          new FileType("Unknown", "",
          "i-regular.png");
  public static FileType JAVA_TYPE =
          new FileType("Java Source", "java",
          "gnome-text-x-java.png");
  public static FileType CLASS_TYPE =
          new FileType("Java Byte Code", "class",
          "gnome-application-x-java-byte-code.png");
  public static FileType JAR_TYPE =
          new FileType("Jar Archive", "jar",
          "gnome-application-x-jar.png");
  public static FileType SH_TYPE =
          new FileType("Bourne Shell Script", "sh",
          "gnome-text-x-sh.png");
  public static FileType CSH_TYPE =
          new FileType("C Shell Script", "csh",
          "gnome-text-x-csh.png");
  public static FileType GIF_TYPE =
          new FileType("GIF Image", "gif",
          "gnome-image-gif.png");
  public static FileType JPEG_TYPE =
          new FileType("JPEG Image", "jpeg jpg",
          "gnome-image-jpeg.png");
  public static FileType PGM_TYPE =
          new FileType("PGM Image", "pgm",
          "gnome-image-pgm.png");
  public static FileType HTML_TYPE =
          new FileType("HTML", "html htm",
          "gnome-text-html.png");
  public static FileType XML_TYPE =
          new FileType("XML", "xml",
          "gnome-text-xml.png");
  public static FileType PS_TYPE =
          new FileType("PostScript", "ps",
          "gnome-application-postscript.png");
  public static FileType SQL_TYPE =
          new FileType("SQL", "sql",
          "gnome-text-x-sql.png");
  public static FileType TEX_TYPE =
          new FileType("TeX", "tex",
          "gnome-tex.png");

  public static FileType getType(File f) {
    if (f.isDirectory()) {
      return FOLDER_TYPE;
    }
    if (fileTypes == null) {
      return UNKNOWN_TYPE;
    }
    String name = f.getName();
    int dotIndex = name.lastIndexOf(".");
    if (dotIndex == -1) {
      return UNKNOWN_TYPE;
    }
    String suffix = name.substring(dotIndex + 1);
    FileType fileType = (FileType) fileTypes.get(suffix);
    if (fileType == null) {
      return UNKNOWN_TYPE;
    }
    return fileType;
  }

  private String name;
  private Icon icon;
  private String[] suffixArray;
  private FileFilter filter;

  private void addSuffixes(String sfx) {
    if (sfx == null || sfx.length() == 0) {
      return;
    }
    // Parse the list of suffixes
    suffixArray = sfx.split("\\s+");
    // Create the fileTypes map if necessary
    if (fileTypes == null) {
      fileTypes = new HashMap<String, FileType>();
    }
    // Register the file type
    for (int i = 0; i < suffixArray.length; i++) {
      String suffix = suffixArray[i];
      if (suffix.length() > 0) {
        fileTypes.put(suffix, this);
      }
    }
  }

  /**
   *  This constructor is disabled.
   *  A FileType must have a type name, suffixes, and an icon name.
   */
  private FileType() {
  }

  /**
   *  new FileType(tnm, sfx, inm) returns a new FileType with type name
   *  tnm, suffixes sfx, and icon name inm.
   */
  public FileType(String tnm, String sfx, String inm) {
    name = tnm;
    addSuffixes(sfx);
    icon = IconManager.getIcon(inm);
  }

  /**
   *  ft.getIcon() returns the icon for ft.
   */
  public Icon getIcon() {
    return icon;
  }

  /**
   *  ft.getFilter() returns the file filter for ft.
   */
  public FileFilter getFilter() {
    if (filter == null) {
      filter = new FileFilter() {

        public boolean accept(File f) {
          String fileName = f.getName();
          for (int i = 0; i < suffixArray.length; i++) {
            if (fileName.endsWith(suffixArray[i])) {
              return true;
            }
          }
          return false;
        }

      };
    }
    return filter;
  }

}  // public class FileType

