package gui;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.Vector;
import javax.imageio.ImageIO;
import javax.swing.filechooser.FileFilter;

/**
 *
 * @author gshute
 */
public class ImageType {

  private static SortedMap<String, ImageType> types;
  private static FileFilter filter;
  private String name;

  private static BufferedImage getImage(Component c) {
    int width = c.getWidth();
    int height = c.getHeight();
    BufferedImage image =
            new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
    Graphics graphics = image.getGraphics();
    c.paint(graphics);
    return image;
  }

  private static String getSuffix(File f) {
    String name = f.getName();
    int dotPosition = name.lastIndexOf(".");
    if (dotPosition < 0) {
      return null;
    }
    String suffix = name.substring(dotPosition + 1);
    return suffix.toLowerCase();
  }

  private static void addType(ImageType t) {
    types.put(t.name, t);
  }

  private static void createImageTypes() {
    types = new TreeMap<String, ImageType>();
//      String[] suffixes = ImageIO.getWriterFileSuffixes();
//      for (int i = 0; i < suffixes.length; i++) {
//        addType(new ImageType(suffixes[i]));
//      }
    addType(new ImageType("bmp"));
    addType(new ImageType("gif"));
    addType(new ImageType("jpeg"));
    addType(new ImageType("jpg"));
    addType(new ImageType("png"));
    EPSImageType epsImageType = new EPSImageType();
    if (epsImageType.isAvailable()) {
      addType(epsImageType);
    }
    filter = new FileFilter() {

      public boolean accept(File f) {
        if (f.isDirectory()) {
          return true;
        }
        String suffix = getSuffix(f);
        if (suffix == null) {
          return false;
        }
        return types.containsKey(suffix);
      }

      public String getDescription() {
        return "Image Files";
      }

    };
  }

  public ImageType(String nm) {
    name = nm;
  }

  public static Vector<ImageType> getTypes() {
    if (types == null) {
      createImageTypes();
    }
    return new Vector<ImageType>(types.values());
  }

  public static FileFilter getFileFilter() {
    if (types == null) {
      createImageTypes();
    }
    return filter;
  }

  public static ImageType getFileType(File f) {
    String suffix = getSuffix(f);
    if (suffix == null) {
      return null;
    }
    return types.get(getSuffix(f));
  }

  public File changeSuffix(File f) {
    String path = f.getAbsolutePath();
    int dotPosition = path.lastIndexOf(".");
    if (dotPosition >= 0) {
      path = path.substring(0, dotPosition);
    }
    return new File(path + "." + name);
  }

  public void write(File f, Component c)
          throws IOException {
    BufferedImage image = getImage(c);
    ImageIO.write(image, name, f);
  }

  public String toString() {
    return name;
  }

}  // public class ImageType
