package gui.multisource;

import gui.Viewable;
import java.awt.GridLayout;
import java.util.Enumeration;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.text.AbstractDocument;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeSelectionModel;

/**
 * An ElementView provides the capability of displaying the
 * element structure and attributes of a document or part of
 * a document.
 */
public class ElementView
implements Viewable {

  private JTree tree;
  private DefaultTreeModel treeModel;
  private JTextArea textArea;
  private JPanel panel;

  private void displayAttributes(AbstractDocument.AbstractElement e) {
    textArea.setText("");
    if (e == null) {
      return;
    }
    Enumeration<?> enumeration = e.getAttributeNames();
    while (enumeration.hasMoreElements()) {
      Object name = enumeration.nextElement();
      Object value = e.getAttribute(name);
      textArea.append("" + name + ": " + value + "\n");
    }
    displayAttributes(
            (AbstractDocument.AbstractElement) e.getResolveParent());
  }

  /**
   * new ElementView() returns a new ElementView.
   */
  public ElementView() {
    tree = new JTree(treeModel);
    treeModel = (DefaultTreeModel) tree.getModel();
    tree.getSelectionModel().setSelectionMode(
            TreeSelectionModel.SINGLE_TREE_SELECTION);
    textArea = new JTextArea();
    tree.addTreeSelectionListener(new TreeSelectionListener() {

      public void valueChanged(TreeSelectionEvent ev) {
        Object element =
                tree.getSelectionPath().getLastPathComponent();
        AbstractDocument.AbstractElement selectedElement =
                (AbstractDocument.AbstractElement) element;
        displayAttributes(selectedElement);
      }

    });
    panel = new JPanel(new GridLayout(1, 2));
    panel.add(tree);
    panel.add(textArea);
  }

  /**
   * ev.setRoot() sets r as the root element for ev's view.
   */
  public void setRoot(AbstractDocument.AbstractElement r) {
    treeModel.reload(r);
  }

  /**
   * ev.refresh() refreshes ev's view of its root element.
   */
  public void refresh() {
    treeModel.reload();
  }

  /**
   * ev.getTree() returns the tree component for ev.
   */
  public JTree getTree() {
    return tree;
  }

  /**
   * ev.getViewComponent() returns the panel component for ev.
   * This component contains both ev's tree and its attribute
   * text area.
   */
  public JPanel getViewComponent() {
    return panel;
  }

}  // public class ElementView
