/*
 * CompositeModel.java
 *
 * Created on August 13, 2006, 5:20 PM
 *
 */
package mvc;

/**
 *  A CompositeModel gets its value from its currently selected submodel.
 *
 *  @author gshute
 */
public class CompositeModel<T, S extends NamedModel<? extends T>>
        extends CallbackExecutor
        implements NamedModel<T> {

  private String name;
  private Choice.Model<S> submodelModel;
  private S selectedSubmodel;
  private Callback valueChangeCallback;

  /**
   *  This constructor is disabled.
   *  A CompositeModel must have a name and a modelChoice for submodels.
   */
  private CompositeModel() {
  }

  /**
   *  new CompositeModel(nm, mc) returns a new CompositeModel named nm.
   *  It gets its value from its currently selected submodel.
   *  Its submodel model is constructed from the choices in mc.
   */
  public CompositeModel(String nm, Choice<S> mc) {
    this(nm, mc.createModel(nm));
  }

  /**
   *  new CompositeModel(nm, smm) returns a new CompositeModel named nm.
   *  It gets its value from its currently selected submodel.
   *  Its submodel model is smm.
   */
  public CompositeModel(String nm, Choice.Model<S> smm) {
    name = nm;
    submodelModel = smm;
    selectedSubmodel = submodelModel.getValue();
    valueChangeCallback = new Callback() {

      public void execute() {
        executeCallbacks();
      }

    };
    selectedSubmodel.addCallback(valueChangeCallback);
    Callback submodelChangeCallback = new Callback() {

      public void execute() {
        selectedSubmodel.removeCallback(valueChangeCallback);
        selectedSubmodel = submodelModel.getValue();
        selectedSubmodel.addCallback(valueChangeCallback);
        executeCallbacks();
      }

    };
    submodelModel.addCallback(submodelChangeCallback);
  }

  /**
   *  m.getName() returns the name of m.
   * @return 
   */
  public String getName() {
    return name;
  }

  /**
   *  m.getValue() returns the current value of m.
   * @return 
   */
  public T getValue() {
    return selectedSubmodel.getValue();
  }

  /**
   *  m.getSelectedModel() returns the current selected submodel of m.
   * @return 
   */
  public S getSelectedSubmodel() {
    return selectedSubmodel;
  }

  /**
   *  m.getSelectedModel() returns the current selected submodel of m.
   * @return 
   */
  public Choice.Model<S> getSubmodelModel() {
    return submodelModel;
  }

}  // public class CompositeModel
