/*
 * CompositeNotifier.java
 *
 * Created on July 26, 2006, 9:34 AM
 *
 */
package mvc;

import java.util.ArrayList;
import java.util.List;

/**
 *  A CompositeNotifier is a notifier that manages notifications
 *  for a composite model.
 *  It provides methods for adding and removing submodels of a
 *  composite model.
 *  Callbacks registered with the composite notifier are notified
 *  when the value changes for any submodel.
 *  They are not automatically notified when submodels are added
 *  or removed.
 *  Clients can send an executeCallbacks() message to the composite
 *  notifier when submodel changes have reached a stable state.
 * 
 *  @author gshute
 */
public class CompositeNotifier
        extends CallbackExecutor {

  private List<NamedModel<?>> submodels;
  private Callback submodelCallback;

  /**
   *  new CompositeNotifier() returns a new CompositeNotifier.
   */
  public CompositeNotifier() {
    submodels = new ArrayList<NamedModel<?>>();
    submodelCallback = new Callback() {

      public void execute() {
        executeCallbacks();
      }

    };
  }

  /**
   *  DOCME.
   * @param m 
   */
  public void addSubmodel(NamedModel<?> m) {
    submodels.add(m);
    m.addCallback(submodelCallback);
  }

  /**
   *  DOCME.
   * @param m 
   */
  public void removeSubmodel(NamedModel<?> m) {
    m.removeCallback(submodelCallback);
    submodels.remove(m);
  }

}  // public class CompositeNotifier
