/*
 * ConstantModel.java
 *
 * Created on August 13, 2006, 12:53 PM
 *
 */
package mvc;

/**
 *  A ConstantModel is a NamedModel whose value is fixed when it is
 *  constructed.
 *
 *  @author gshute
 */
public class ConstantModel<T>
        implements NamedModel<T> {

  String name;
  T value;

  /**
   *  This constructor is disabled.
   *  A ConstantModel must have a name and a value.
   */
  private ConstantModel() {
  }

  /**
   *  new ConstantModel(nm, val) returns a new ConstantModel
   *  with name nm and constant value val.
   */
  public ConstantModel(String nm, T val) {
    name = nm;
    value = val;
  }

  /**
   *  m.getName() returns the name for m.
   * @return 
   */
  public String getName() {
    return name;
  }

  /**
   *  m.getValue() returns the current value of m.
   *  For a ConstantModel, the model value is always the value provided
   *  in the constructor.
   * @return the current value
   */
  public T getValue() {
    return value;
  }

  /**
   *  m.addCallback(cbk) registers cbk as an callback of m.
   *  For a ConstantModel, the model value never changes so callback
   *  registration is ignored.
   * @param cbk the callback to be added
   */
  public void addCallback(Callback cbk) {
  }

  /**
   *  m.removeCallback(cbk) unregisters cbk as an callback of m.
   *  For a ConstantModel, the model value never changes so callback
   *  registration is ignored.
   * @param cbk the callback to be removed
   */
  public void removeCallback(Callback cbk) {
  }

}  // public class ConstantModel
