/*
 * DerivedModelBase.java
 *
 * Created on August 6, 2006, 4:24 PM
 *
 */
package mvc;

import java.util.ArrayList;
import java.util.List;

/**
 *  A DerivedModelBase provides a registration, unregistration and
 *  execution service for callbacks.
 * 
 *  @author gshute
 */
public abstract class DerivedModelBase<T>
        implements Model<T> {

  private List<Callback> callbackList;
  private List<Model<?>> sourceModels;
  private Callback sourceModelCallback;
  private T value;

  /**
   *  ce.executeCallbacks() sends an execute() message to
   *  all callbacks that are registered with ce.
   */
  private void executeCallbacks() {
    for (Callback callback : callbackList) {
      callback.execute();
    }
  }
  
  protected void addSourceModel(Model<?> m) {
    sourceModels.add(m);
    m.addCallback(sourceModelCallback);
  }
  
  protected abstract T computeValue();

  /**
   *  new DerivedModelBase() returns a new DerivedModelBase.
   */
  public DerivedModelBase() {
    callbackList = new ArrayList<Callback>();
    sourceModels = new ArrayList<Model<?>>();
    sourceModelCallback = new Callback() {
      public void execute() {
//        value = computeValue();
        executeCallbacks();
      }
    };
  }
  
  public final T getValue() {
//    return value;
    return computeValue();
  }

  /**
   *  ce.addCallback(cbk) registers cbk as an callback that is
   *  executed by ce when ce.executeCallbacks() is invoked.
   * @param cbk the callback to be registered
   */
  public void addCallback(Callback cbk) {
    callbackList.add(cbk);
  }

  /**
   *  ce.removeCallback(cbk) unregisters cbk as an callback
   *  that is executed by ce when ce.executeCallbacks() is
   *  invoked.
   * @param cbk the callback to be unregistered
   */
  public void removeCallback(Callback cbk) {
    callbackList.remove(cbk);
  }

}  // public class DerivedModelBase
