/*
 * IntegralRange.java
 *
 * Created on August 8, 2006, 5:06 PM
 *
 */
package mvc;

import gui.Viewable;
import javax.swing.JComponent;

/**
 *  An IntegralRange is a model type for integer values, specifying lower
 *  and upper bounds on values.
 *
 *  @author gshute
 */
public class IntegralRange {

  /**
   *  DOCME.
   */
  public static class Model
          extends NamedDirectModel<Integer> {

    int value;
    IntegralRange range;

    /**
     *  DOCME.
     */
    private Model(String nm, IntegralRange r) {
      super(nm);
      range = r;
      value = range.getStart();
    }

    /**
     *  DOCME.
     * @return 
     */
    public Integer getValue() {
      return new Integer(value);
    }

    /**
     *  DOCME.
     * @return 
     */
    public int getIntValue() {
      return value;
    }

    /**
     *  DOCME.
     * @return 
     */
    public IntegralRange getRange() {
      return range;
    }

    /**
     *  DOCME.
     * @param val 
     */
    public void setIntValue(int val) {
      value = val;
      notifyObservers();
    }

  }  // public static class IntegralRange.Model

  /**
   *  DOCME.
   */
  public static abstract class Controller
          implements Viewable {

    private JComponent viewComponent;
    private UpdateManager<Integer, IntegralRange.Model> updateManager;

    /**
     *  DOCME.
     * @return 
     */
    protected abstract JComponent createViewComponent();

    /**
     *  DOCME.
     * @param lo 
     * @param hi 
     */
    protected abstract void setBounds(int lo, int hi);

    /**
     *  DOCME.
     * @param val 
     */
    protected abstract void setViewValue(int val);

    /**
     *  DOCME.
     * @return 
     */
    protected abstract int getViewValue();

    /**
     *  DOCME.
     */
    protected final void setModelValue() {
      updateManager.setModelValue();
    }

    /**
     *  new Controller(m) returns a new Controller for the
     *  values of m.
     * @param m 
     */
    public Controller(IntegralRange.Model m) {
      viewComponent = createViewComponent();
      Callback updateModelResponder = new Callback() {

        public void execute() {
          getModel().setIntValue(getViewValue());
        }

      };
      Callback updateViewResponder = new Callback() {

        public void execute() {
          setViewValue(getModel().getIntValue());
        }

      };
      Callback replaceModelResponder = new Callback() {

        public void execute() {
          IntegralRange range = getModel().getRange();
          setBounds(range.getLow(), range.getHigh());
        }

      };
      updateManager = new UpdateManager<Integer, IntegralRange.Model>(m,
              updateModelResponder, replaceModelResponder, updateViewResponder);
      setModel(m);
    }

    /**
     *  DOCME.
     * @return
     */
    public JComponent getViewComponent() {
      return viewComponent;
    }

    /**
     *  DOCME.
     * @param m
     */
    public void setModel(IntegralRange.Model m) {
      updateManager.setModel(m);
    }

    /**
     *  DOCME.
     * @return
     */
    public IntegralRange.Model getModel() {
      return updateManager.getModel();
    }

  }  // public static abstract class Controller<TE>

  Integer low;
  Integer high;
  Integer start;

  /**
   *  DOCME.
   */
  private IntegralRange() {
  }

  /**
   *  new IntegralRange(lo, hi) returns a new IntegralRange with values from
   *  lo to hi.
   *  The start value for the range is lo.
   * @param lo 
   * @param hi 
   */
  public IntegralRange(Integer lo, Integer hi) {
    this(lo, hi, lo);
  }

  /**
   *  new IntegralRange(lo, hi, st) returns a new IntegralRange with values
   *  from lo to hi.
   *  The start value for the range is st.
   * @param lo 
   * @param hi 
   * @param st 
   */
  public IntegralRange(Integer lo, Integer hi, Integer st) {
    if (lo > hi) {
      throw new IllegalArgumentException("Empty range");
    }
    if (st < lo || hi < st) {
      throw new IllegalArgumentException("Start value out of range");
    }
    low = lo;
    high = hi;
    start = st;
  }

  /**
   *  mc.getStart() returns the starting value for values of
   *  type mc.
   * @return 
   */
  public int getStart() {
    return start;
  }

  /**
   *  r.getLow() returns the lower bound on values of type r.
   * @return 
   */
  public int getLow() {
    return low;
  }

  /**
   *  r.getHigh() returns the upper bound on values of type r.
   * @return 
   */
  public int getHigh() {
    return high;
  }

  /**
   *  mc.createModel() returns a new editable model whose possible
   *  values are the choices of mc.
   * @param nm 
   * @return 
   */
  public Model createModel(String nm) {
    return new IntegralRange.Model(nm, this);
  }

}  // public class IntegralRange

