/*
 * NamerView.java
 *
 * Created on July 26, 2006, 10:06 AM
 *
 */
package mvc;

import gui.NamedViewable;
import javax.swing.JLabel;
import conversion.Namer;

/**
 *  A NamerView displays the value of a model using a
 *  {@link Namer} to convert the value to a string.
 * 
 *  @author gshute
 */
public class NamerView<T>
        implements NamedViewable, View<T, NamedModel<? extends T>> {

  private JLabel label;
  private Callback callback;
  private NamedModel<? extends T> model;
  private Namer<? super T> namer;

  /**
   *  setLabelText() sets the label text for this namer view
   *  from the current value of its model.
   *  The name is determined by the namer variable.
   */
  private void setLabelText() {
    label.setText(namer.getName(model.getValue()));
  }

  /**
   *  This constructor is disabled.
   */
  private NamerView() {
  }

  /**
   *  new NamerView(m) returns a new NamerView of model m
   *  using an {@link ObjectNamer} as a namer.
   * @param m 
   */
  public NamerView(NamedModel<? extends T> m) {
    this(m, new ObjectNamer());
  }

  /**
   *  new NamerView(m, nmr) returns a new NamerView of model m
   *  using nmr as a namer.
   * @param m 
   * @param nmr 
   */
  public NamerView(NamedModel<? extends T> m, Namer<? super T> nmr) {
    label = new JLabel("", JLabel.CENTER);
    callback = new Callback() {

      public void execute() {
        setLabelText();
      }

    };
    namer = nmr;
    setModel(m);
  }

  /**
   *  nv.getName() returns the name of the model that mv is
   *  viewing.
   * @return 
   */
  public String getName() {
    return model.getName();
  }

  /**
   *  nv.getViewComponent() returns the view component for nv.
   * @return 
   */
  public JLabel getViewComponent() {
    return label;
  }

  /**
   *  nv.setModel(m) arranges for vw to view the value of m.
   * @param m 
   */
  public void setModel(NamedModel<? extends T> m) {
    if (m != null) {
      m.removeCallback(callback);
    }
    model = m;
    if (model != null) {
      model.addCallback(callback);
    }
    setLabelText();
  }

}  // public class NamerView<T>
