/*
 * RealRange.java
 *
 * Created on August 8, 2006, 5:06 PM
 *
 */
package mvc;

import gui.Viewable;
import javax.swing.JComponent;

/**
 *  A RealRange is a model type for double values, specifying lower
 *  and upper bounds on values.
 *
 *  @author gshute
 */
public class RealRange {

  /**
   *  DOCME.
   */
  public static class Model
          extends NamedDirectModel<Double> {

    double value;
    RealRange range;

    /**
     *  DOCME.
     */
    private Model(String nm, RealRange r) {
      super(nm);
      range = r;
      value = range.getStart();
    }

    /**
     *  DOCME.
     * @return 
     */
    public Double getValue() {
      return new Double(value);
    }

    /**
     *  DOCME.
     * @return 
     */
    public double getDoubleValue() {
      return value;
    }

    /**
     *  DOCME.
     * @return 
     */
    public RealRange getRange() {
      return range;
    }

    /**
     *  DOCME.
     * @param val 
     */
    public void setDoubleValue(double val) {
      value = val;
      notifyObservers();
    }

  }  // public static class RealRange.Model

  /**
   *  DOCME.
   */
  public static abstract class Controller
          implements Viewable {

    private JComponent viewComponent;
    private UpdateManager<Double, RealRange.Model> updateManager;

    /**
     *  DOCME.
     * @return 
     */
    protected abstract JComponent createViewComponent();

    /**
     *  DOCME.
     * @param lo 
     * @param hi 
     */
    protected abstract void setBounds(double lo, double hi);

    /**
     *  DOCME.
     * @param val 
     */
    protected abstract void setViewValue(double val);

    /**
     *  DOCME.
     * @return 
     */
    protected abstract double getViewValue();

    /**
     *  DOCME.
     */
    protected final void setModelValue() {
      updateManager.setModelValue();
    }

    /**
     *  new Controller(m) returns a new Controller for the
     *  values of m.
     * @param m 
     */
    public Controller(RealRange.Model m) {
      viewComponent = createViewComponent();
      Callback updateModelResponder = new Callback() {

        public void execute() {
          getModel().setDoubleValue(getViewValue());
        }

      };
      Callback updateViewResponder = new Callback() {

        public void execute() {
          setViewValue(getModel().getDoubleValue());
        }

      };
      Callback replaceModelResponder = new Callback() {

        public void execute() {
          RealRange range = getModel().getRange();
          setBounds(range.getLow(), range.getHigh());
        }

      };
      updateManager = new UpdateManager<Double, RealRange.Model>(m,
              updateModelResponder, replaceModelResponder, updateViewResponder);
      setModel(m);
    }

    /**
     *  DOCME.
     * @return
     */
    public JComponent getViewComponent() {
      return viewComponent;
    }

    /**
     *  DOCME.
     * @param m
     */
    public void setModel(RealRange.Model m) {
      updateManager.setModel(m);
    }

    /**
     *  DOCME.
     * @return
     */
    public RealRange.Model getModel() {
      return updateManager.getModel();
    }

  }  // public static abstract class Controller<TE>

  Double low;
  Double high;
  Double start;

  /**
   *  DOCME.
   */
  private RealRange() {
  }

  /**
   *  new RealRange(lo, hi) returns a new RealRange with values from
   *  lo to hi.
   *  The start value for the range is lo.
   * @param lo 
   * @param hi 
   */
  public RealRange(Double lo, Double hi) {
    this(lo, hi, lo);
  }

  /**
   *  new RealRange(lo, hi, st) returns a new RealRange with values
   *  from lo to hi.
   *  The start value for the range is st.
   * @param lo 
   * @param hi 
   * @param st 
   */
  public RealRange(Double lo, Double hi, Double st) {
    if (lo > hi) {
      throw new IllegalArgumentException("Empty range");
    }
    if (st < lo || hi < st) {
      throw new IllegalArgumentException("Start value out of range");
    }
    low = lo;
    high = hi;
    start = st;
  }

  /**
   *  mc.getStart() returns the starting value for values of
   *  type mc.
   * @return 
   */
  public double getStart() {
    return start;
  }

  /**
   *  r.getLow() returns the lower bound on values of type r.
   * @return 
   */
  public double getLow() {
    return low;
  }

  /**
   *  r.getHigh() returns the upper bound on values of type r.
   * @return 
   */
  public double getHigh() {
    return high;
  }

  /**
   *  mc.createModel() returns a new editable model whose possible
   *  values are the choices of mc.
   * @param nm 
   * @return 
   */
  public Model createModel(String nm) {
    return new RealRange.Model(nm, this);
  }

}  // public class RealRange
