package mvc;

import gui.AbstractSizer;
import java.awt.Dimension;
import java.awt.Rectangle;
import javax.swing.JTabbedPane;

/**
 * A Tabber is just the tab part of a JTabbedPane.
 * It can be used to build a chooser without needing to
 * install a component for each choice.
 * This essentially separates views and controllers
 * in a Model-View-Controler pattern.
 * 
 * @author gshute
 */
public class Tabber
        extends JTabbedPane {

  private GlueFactory glueFactory;
  private AbstractSizer sizer;

  /**
   * new Tabber() returns a new tabber.
   */
  public Tabber() {
    super();
    glueFactory = new GlueFactory();
    sizer = new AbstractSizer(this) {

      public Dimension computePreferredSize(Dimension sz) {
        Rectangle tabRectangle = getBoundsAt(0);
        int height = tabRectangle.getSize().height + tabRectangle.y + 1;
        return new Dimension(sz.width, height);
      }

    };
  }

  /**
   * t.setWidth() adjusts the height of t to fit its
   * current width.
   */
  public void setWidth() {
    setWidth(getWidth());
  }

  /**
   * t.setWidth(w) adjusts the width and height of t
   * to give it width w.
   * 
   * @param w the new width
   */
  public void setWidth(int w) {
    sizer.setWidth(w);
  }

  /**
   * t.removeAll() removes all titles displayed by t.
   */
  public void removeAll() {
    super.removeAll();
    glueFactory.freeGlues();
  }

  /**
   * t.add(ttl) adds a tab with title ttl to t.
   * 
   * @param ttl
   */
  public void add(String ttl) {
    super.add(ttl, glueFactory.createGlue());
  }

}  // public class Tabber
