/*
 * UpdateManager.java
 *
 * Created on December 18, 2006, 8:00 PM
 *
 */
package mvc;

/**
 *  An UpdateManager manages update communication between
 *  a controller and its current model.
 *
 *  @author gshute
 */
public class UpdateManager<T, TM extends NamedModel<? extends T>> {

  private TM model;
  private Callback modelResponder;
  private Callback updateModelCallback;
  private Callback replaceModelCallback;
  private Callback updateViewCallback;
  private boolean updating;

  /**
   *  This contructor is disabled.
   */
  private UpdateManager() {
  }

  /**
   *  new UpdateManager(m, umc, rmc, uvc) returns a new update manager
   *  for a controller that interacts with m.
   *  An update manager will normally be constructed by a controller
   *  that provides the callbacks umc, rmc, and uvc.
   * @param m the model whose updates are to be controlled
   * @param umc the callback for updating the model value
   * @param rmc the callback for replacing the model with another model
   * @param uvc the callback for updating the controller's view
   */
  public UpdateManager(TM m, Callback umc, Callback rmc, Callback uvc) {
    model = m;
    updateModelCallback = umc;
    replaceModelCallback = rmc;
    updateViewCallback = uvc;
    modelResponder = new Callback() {

      public void execute() {
        if (!updating) {
          updating = true;
          updateViewCallback.execute();
          updating = false;
        }
      }

    };
  }

  /**
   *  um.setModel(m) replaces um's current model with m.
   * @param m the new model
   */
  public void setModel(TM m) {
    if (model != null) {
      model.removeCallback(modelResponder);
    }
    model = m;
    updating = true;
    replaceModelCallback.execute();
    updating = false;
    modelResponder.execute();
    model.addCallback(modelResponder);
  }

  /**
   *  um.getModel() returns um's current model.
   * @return the current model
   */
  public TM getModel() {
    return model;
  }

  /**
   *  um.setModelValue() updates um's current model if necessary.
   */
  public void setModelValue() {
    if (!updating) {
      updating = true;
      updateModelCallback.execute();
      updating = false;
    }
  }

}  // public class UpdateManager<T, TM extends NamedModel<? extends T>>
