package vfs;

import java.io.File;
import java.io.IOException;
import java.net.URL;

/**
 * A NativeDirectory is a VFSDirectory representing
 * a directory in the native file system.
 *
 * @author gshute
 */
public class NativeDirectory
        extends VFSDirectory {

  File path;

  /**
   * populate() fills in this NativeDirectory with
   * VFSFile and VFSDirectory objects constructed from
   * files and subdirectories of the directory named
   * by the instance variable path in the underlying
   * file system.
   */
  private void populate()
          throws IOException {
    File[] files = path.listFiles();
    for (File file : files) {
      String name = file.getName();
      if (file.isDirectory()) {
        addSubdirectory(new NativeDirectory(name + "/", this));
      } else {
        addFile(name);
      }
    }
  }

  /**
   * new NativeDirectory(url) returns a new virtual file
   * system root directory representing the native directory
   * whose URL is url.
   * 
   * @param url the URL of the native root directory
   */
  public NativeDirectory(URL url) {
    super(url);
    try {
      path = new File(url.toURI());
      populate();
    } catch (Exception ex) {
      throw new VFSError(ex);
    }
  }

  /**
   * new NativeDirectory(nm, par) returns a virtual directory
   * named nm whose parent is par.
   * 
   * @param nm the name of the native directory
   * @param par its virtual file system parent
   */
  public NativeDirectory(String nm, NativeDirectory par) {
    super(nm, par);
    try {
      path = new File(par.path, nm);
      populate();
    } catch (IOException ex) {
      throw new VFSError(ex);
    }
  }

}  // public class NativeDirectory
